﻿Imports System.Text
Imports Newtonsoft.Json.Linq

Friend Class Connector

    Public AccessType As String = "WSDL"
    Public ErrorMessage As String = ""

    Private w As New xAssets.WebService

    Private _hash As String = ""

    Private _username As String
    Private _password As Security.SecureString

    '   Get local ip address (web facing would be better, and could be coded by visiting 8.8.8.8 and then getting the endpoint ip)
    Private ReadOnly Property _ip As String = Array.FindAll(System.Net.Dns.GetHostEntry(String.Empty).AddressList, Function(a) a.AddressFamily = System.Net.Sockets.AddressFamily.InterNetwork).FirstOrDefault().ToString

    Private _db As String = "x7"
    Private _dns As String = "x7"
    Private _port As Integer = 58978   ' 443 for ssl
    Private _scheme As String = "https"
    Private _subdir As String = ""
    Private _nonce As String = ""
    Private _noncedate As Date = Now
    Private _apikey As String = ""

    Private _addapikey As Boolean = False

    ''' <summary>
    ''' Returns a flag indicating that the current session is valid and the user is logged on
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property Loggedon As Boolean
        Get
            If AccessType = "WSDL" Then
                Return _hash <> ""      ' xAssets api hash
            Else
                Return _hash <> ""      ' bearer token
            End If
        End Get
    End Property

    Private ReadOnly Property Path As String
        Get
            Dim url As String = _scheme & "://" & _dns
            If _port > 0 Then
                url &= ":" & _port
            End If

            If _subdir <> "" Then
                url &= "/" & _subdir
            End If

            Return url
        End Get
    End Property

    ''' <summary>
    ''' Returns the path needed to collect local data
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property DataPath As String
        Get

            Dim url As String = Me.Path & "/customerdata/" & _db
            Return url

        End Get
    End Property

    ''' <summary>
    ''' Connect to the specified instance with the username and password provided
    ''' </summary>
    ''' <param name="username"></param>
    ''' <param name="password"></param>
    Public Sub Connect(username As String, password As System.Security.SecureString)

        Try
            ErrorMessage = ""

            'w.PreAuthenticate = False
            'w.UseDefaultCredentials = True

            If AccessType = "REST" Then

                _username = username            ' username is the API key
                _password = Nothing

                w.Url = Me.Path & "/api/api.ashx?apikey=" & username & "&database=" & _db & "&command=apilogon"

                Logon(Nothing)

            Else

                _username = username
                _password = password

                '   Connect to our hosted logon '  "http://x7:58978/webservice.asmx"

                w.Url = Me.Path & "/webservice.asmx"

                Logon(password)
            End If

        Catch ex As Exception
            ErrorMessage = ex.Message
        End Try


    End Sub

    Private Sub Logon(password As Security.SecureString)
        '   Invoke the logon itself

        If AccessType = "rest" Then
            Dim web As New System.Net.WebClient()
            Dim json As String
            web.Headers.Add("User-Agent", "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:25.0) Gecko/20100101 Firefox/25.0")
            Try
                json = web.DownloadString(w.Url)
            Catch ex As System.Net.WebException
                Dim r As System.Net.WebResponse = ex.Response
                If r IsNot Nothing Then
                    r.GetResponseStream()
                    Dim responseText As String
                    Using reader As New System.IO.StreamReader(r.GetResponseStream(), Encoding.UTF8)
                        responseText = reader.ReadToEnd()
                    End Using
                    Try
                        System.IO.File.WriteAllText(My.Application.Info.DirectoryPath & "\lasterror.htm", ex.Message & " " & responseText)
                    Catch
                    End Try
                End If

                Throw ex
            End Try

            Dim o As JObject = JObject.Parse(json)

            Dim e As String = GetResultFromJson(o, "error")
            If e <> "" Then
                Throw New Exception(e)
            End If

            _hash = GetResultFromJson(o, "hash")
            _nonce = GetResultFromJson(o, "nonce")
            _noncedate = CDate(GetResultFromJson(o, "noncetime"))

            If _hash = "" Then
                Throw New Exception("Unexpected error - a hash was not returned from the API logon process")
            End If

        Else
            Dim passwordstring As String = GetPassword(password)
            'If passwordstring = "$challengeresponse$" Then
            '    System.Security.Principal.WindowsIdentity.GetCurrent()
            '    w.Credentials = System.Net.CredentialCache.DefaultCredentials
            'End If
            _hash = w.WebLogonCollectionServer(_username, passwordstring, _db, _ip, "api", _dns, _port, _scheme, _nonce, _noncedate)
        End If
    End Sub

    Private Function GetPassword(password As Security.SecureString) As String
        Return New System.Net.NetworkCredential(String.Empty, password).Password
    End Function

    ''' <summary>
    ''' Run a query optionally with parameters and return the xml or json generated by the query
    ''' </summary>
    ''' <param name="query">The SavedSelectID or queryname of the query</param>
    ''' <param name="parameters">For queries which require parameters, place the parameters string in the form p?arg0=xxx%2526arg1=yyyy%2526arg2=zzz</param>
    ''' <returns>an XML string with query data</returns>
    Public Function QueryXML(query As String, parameters As String) As String

        Try

            ErrorMessage = ""

            Dim queryparams() As String = {query, "", "", "", parameters}

            If Me.AccessType = "rest" Then
                Return Restful("", "XML", "commandprocessor", "AssetXML", "", queryparams)
            Else
                Return CommandXML("AssetXML", queryparams)
            End If

        Catch ex As Exception
            ErrorMessage = ex.Message
            Return ""
        End Try

    End Function

    ''' <summary>
    ''' Execute an API command directly using the command and arguments specified
    ''' </summary>
    ''' <param name="command"></param>
    ''' <param name="arguments"></param>
    ''' <returns></returns>

    Public Function CommandXML(command As String, arguments() As String) As String

        Try

            ErrorMessage = ""

            If Me.AccessType = "rest" Then
                Return Restful("", "XML", "commandprocessor", command, "", arguments)
            Else
                Return w.WebCommandProcessorArray(_hash, _username, _db, _ip, command, arguments, _dns, _port, _scheme, _nonce, _noncedate)
            End If

        Catch ex As Exception
            ErrorMessage = ex.Message
            Return ""
        End Try

    End Function

    ''' <summary>
    ''' Save XML back to the database
    ''' </summary>
    ''' <param name="xml"></param>
    ''' <returns></returns>
    Public Function Save(xml As String) As String

        Try

            ErrorMessage = ""

            If Me.AccessType = "rest" Then
                Return Restful("", "XML", "save", "save", xml)
            Else
                Return w.WebSaveXML(_hash, _username, _db, _ip, xml, False, _dns, _port, _scheme, _nonce, _noncedate)
            End If

        Catch ex As Exception
            ErrorMessage = ex.Message
            Return ""
        End Try


    End Function

    ''' <summary>
    ''' Execute a "SaveSpecial" operation to save data to the database
    ''' </summary>
    ''' <param name="command"></param>
    ''' <param name="xml"></param>
    ''' <param name="arguments"></param>
    ''' <returns></returns>
    Public Function SaveSpecial(command As String, xml As String, arguments() As String) As String

        Try

            ErrorMessage = ""

            If Me.AccessType = "rest" Then
                Return Restful("", "XML", "savespecial", command, xml, arguments)
            Else
                Return w.WebSaveSpecial(_hash, _username, _db, _ip, xml, command, arguments, False, _dns, _port, _scheme, _nonce, _noncedate)
            End If

        Catch ex As Exception
            ErrorMessage = ex.Message
            Return ""
        End Try


    End Function

    ''' <summary>
    ''' Execute a restful API call
    ''' </summary>
    ''' <param name="fmt">XML or JSON</param>
    ''' <param name="subcommand">The API command. This can be "save", "savespecial" or a valid CommandProcessor command</param>
    ''' <param name="arguments"></param>
    ''' <returns></returns>
    Public Function Restful(output As String, fmt As String, command As String, subcommand As String, xml As String, ParamArray arguments() As String) As String

        Try

            '   http://x7:58978/api/api.ashx?command=assetxml&username=apikey&database=x7&passwordx=SomePassword!!hh&arg1=10269&arg3=10269

            If output = "" Then output = "returncode"

            Select Case command
                Case "save", "savespecial"
                    Dim url As New StringBuilder(Me.Path)
                    url.Append("/api/api.ashx?command=" & command & "&savespecial=" & subcommand)

                    For i As Integer = 0 To arguments.Length - 1
                        Dim arg As String = arguments(i)
                        If arg <> "" Then
                            url.Append("&arg" & i & "=" & arg)
                        End If
                    Next

                    Dim web As Net.WebClient = PrepareRestfulCall()

                    If _addapikey = True Then
                        AddAPIKey(url)
                    End If

                    Dim args As String = "xml=" & System.Uri.EscapeUriString(xml)

                    'args = "xml=%3Cparams%3E%3Cissuekey%3EPT-3333%3C%2Fissuekey%3E%3C%2Fparams%3E"

                    '   Direct connect in one call

                    Dim s As String = web.UploadString(url.ToString, "POST", args)

                    Dim err As String = GetResultFromJson(s, "error")
                    If err <> "" Then
                        Throw New Exception(err)
                    End If
                    Dim xmlout As String = GetResultFromJson(s, output)      ' could be xml or JSON

                    Return xmlout

                Case "form"
                    Dim url As New StringBuilder(Me.Path)
                    url.Append("/xpanel.aspx?" & arguments(0) & "&apikey=" & _username & "&database=" & _db)
                    Dim web As Net.WebClient = PrepareRestfulCall()
                    Dim s As String = web.DownloadString(url.ToString)
                    Return s

                Case Else
                    Dim url As New StringBuilder(Me.Path)
                    url.Append("/api/api.ashx?command=" & Uri.EscapeDataString(subcommand))

                    For i As Integer = 0 To arguments.Length - 1
                        Dim arg As String = arguments(i)
                        If arg <> "" Then
                            url.Append("&arg" & i & "=" & Uri.EscapeDataString(arg))
                        End If
                    Next

                    If fmt <> "" Then
                        url.Append("&format=" & fmt)
                    End If

                    '   Direct connect in one call

                    Dim web As Net.WebClient = PrepareRestfulCall()

                    If _addapikey = True Then
                        AddAPIKey(url)
                    End If

                    Dim s As String = web.DownloadString(url.ToString)

                    Dim err As String = GetResultFromJson(s, "error")
                    If err <> "" Then
                        Throw New Exception(err)
                    End If
                    Dim xmlout As String = GetResultFromJson(s, output)      ' could be xml or JSON

                    Return xmlout

            End Select

        Catch ex As Exception
            ErrorMessage = ex.Message
            Return ""
        End Try

    End Function

    Private Function PrepareRestfulCall() As Net.WebClient

        Dim web As New System.Net.WebClient()

        If Loggedon = True Then
            web.Headers.Add("Authorization", "Bearer " & _hash)
            web.Headers.Add("Sec", _db & "|" & _nonce & "|" & Format(_noncedate, "dd-MMM-yyyy HH:mm:ss") & "|" & Format(_noncedate, "fff"))
        ElseIf AccessType = "REST" Then
            If _apikey = "" Then
                Throw New Exception("Either logon first or provide an API key")
            End If
            _addapikey = True
        Else
            Throw New Exception("Logon first for WSDL operations")
        End If

        Return web

    End Function

    Private Function GetResultFromJson(o As String, sFind As String) As String

        Dim j As JObject = JObject.Parse(o)
        Return GetResultFromJson(j, sFind)

    End Function

    Private Function GetResultFromJson(o As JObject, sFind As String) As String

        If sFind = "all" Then
            Return o.ToString(Newtonsoft.Json.Formatting.Indented)
        End If

        For Each res As JObject In o.SelectTokens("data[*]")
            If res("type").Value(Of String) = sFind Then
                Return res("value").ToString
            End If
        Next

        Return ""

    End Function

    Private Sub AddAPIKey(url As StringBuilder)

        url.Append("&apikey=" & _apikey & "&database=" & _db)

    End Sub

    Public Sub New(db As String, port As Integer, dns As String, scheme As String, subdir As String, apikey As String)
        _db = db
        _port = port
        _dns = dns
        _scheme = scheme
        _subdir = subdir.Trim("/"c)
        _apikey = apikey
    End Sub



End Class
