﻿Imports Newtonsoft.Json.Linq
Public Class MainForm

    Private c As Connector

    '   Data from server
    Private _xml As String = ""
    Private _assetid As String = ""

    Private Sub EnableControls()
        If InStr(txtUsername.Text, "\") > 0 Then
            txtPassword.Enabled = False
        Else
            txtPassword.Enabled = True
        End If

    End Sub

    '------------------------------------
    '   HANDLERS
    '------------------------------------

    Private Sub Logon_Click(sender As Object, e As EventArgs) Handles Logon.Click

        Try
            Dim sPwd As String
            If InStr(txtUsername.Text, "\") > 0 Then
                sPwd = "$challengeresponse$"
            Else
                sPwd = txtPassword.Text
            End If

            Dim p As New Security.SecureString

            For Each ch As Char In sPwd
                p.AppendChar(ch)
            Next

            c.Connect(txtUsername.Text, p)

            If c.Loggedon = True And c.ErrorMessage = "" Then
                InfoBox.Text = "Logged on OK"
            Else
                InfoBox.Text = "Logon failed : " & c.ErrorMessage
            End If


        Catch ex As Exception
            HandleError(ex)
        End Try
    End Sub

    Private Sub GetAssets_Click(sender As Object, e As EventArgs) Handles GetAssets.Click

        Try
            '   Get all the assets for a specified custodian ID

            Dim paramsxml As String = "p?custodianid=11"

            Dim xml As String = c.QueryXML("Assets With a Specified Custodian", paramsxml)

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = XMLToString(xml)
            End If

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub

    Private Sub SearchCustodian_Click(sender As Object, e As EventArgs) Handles SearchCustodian.Click

        '   Find all custodians with "Owen" in their name

        Try

            Dim paramsxml As String = "p?searchfor=owen"

            Dim xml As String = c.QueryXML("Custodian Search", paramsxml)

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = XMLToString(xml)
            End If

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub

    Private Sub GetOneAsset_Click(sender As Object, e As EventArgs) Handles GetOneAsset.Click

        '   Get the Asset with ID = 24

        Try
            '   Note that 24 could be an IN list such as "24,11" to retrieve two asset records

            Dim xml As String = c.CommandXML("AssetXML", {"EDIT", "2"})

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = XMLToString(xml)
            End If

            _xml = xml
        Catch ex As Exception
            HandleError(ex)
        End Try


    End Sub

    Private Sub ChangeXML_Click(sender As Object, e As EventArgs) Handles ChangeXML.Click

        Try

            '   This changes the _xml string saved in "GetOneAsset", to add a value in the notes field

            Dim notes As String = "Updated by api test " & Now.ToString("dd-MMM-yyyy HH:mm:ss")

            Dim notestag As String = c.CommandXML("FieldTag", {"Notes"})
            If c.ErrorMessage <> "" Then
                Throw New Exception(c.ErrorMessage)
            End If

            Dim doc As New Xml.XmlDocument
            doc.LoadXml(_xml)


            For Each nodrecord As Xml.XmlNode In doc.DocumentElement.ChildNodes

                Dim nodNotes As Xml.XmlNode = nodrecord.SelectSingleNode(notestag)

                Dim text As String = nodNotes.InnerText

                '   If its safe to update then update it

                If text = "" OrElse text.StartsWith("Updated by api test") = True Then
                    nodNotes.InnerText = notes
                End If

            Next

            '   Update the xml private class variable

            _xml = doc.OuterXml

            InfoBox.Text &= vbCrLf & "XML Changed " & vbCrLf & "notes=" & notes & vbCrLf & XMLToString(_xml) & vbCrLf & vbCrLf & XMLToString(_xml)

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub


    Private Sub SaveChanged_Click(sender As Object, e As EventArgs) Handles SaveChanged.Click

        Try
            If _xml <> "" Then

                '   Note that 24 could be an IN list such as "24,11" to retrieve two asset records

                Dim result As String = c.Save(_xml)

                If c.ErrorMessage <> "" Then
                    InfoBox.Text = c.ErrorMessage
                Else
                    InfoBox.Text = "XML saved" & vbCrLf & XMLToString(_xml)
                End If

            End If

        Catch ex As Exception
            HandleError(ex)
        End Try


    End Sub

    Private Sub CreateNewAsset_Click(sender As Object, e As EventArgs) Handles CreateNewAsset.Click

        Try

            '   Easy way to default fields

            Dim xml As String = c.CommandXML("AssetXML", {"NEW", "1",
                                             "DatePurchased", Now.ToString("dd-MMM-yyyy HH:mm:ss"),
                                             "StatusID", "2"})

            If c.ErrorMessage <> "" Then
                Throw New Exception(c.ErrorMessage)
            End If

            '   Harder way is to edit the XML as follows

            Dim doc As New Xml.XmlDocument
            doc.LoadXml(xml)
            Dim rec As Xml.XmlNode = doc.DocumentElement.ChildNodes(0)
            rec.SelectSingleNode("A20").InnerText = "Test Asset Created by API Example Program"
            xml = doc.OuterXml            '   Copy back

            '   Save the xml

            Dim result As String = c.Save(xml)

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                _assetid = result.Substring(8)
                InfoBox.Text = "XML saved:" & result & vbCrLf & "New Asset ID is " & _assetid & vbCrLf & xml
            End If
        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub

    Private Sub DisposeOfAsset_Click(sender As Object, e As EventArgs) Handles DisposeOfAsset.Click

        Try

            Dim x As String =
                            "<BulkDisposal_T>" &
                            "<BulkDisposal_R>" &
                            "<a>1</a>" &
                            "<b>0</b>" &
                            "<D39>" & Now.ToString("dd-MMM-yyyy HH:mm:ss") & "</D39>" &
                            "<FixedList></FixedList>" &
                            "<AID>" & _assetid & "</AID>" &
                            "<SID>" & 6 & "</SID>" &        ' 6 is sold
                            "<BID>1</BID>" &
                            "<B1ID></B1ID>" &
                            "<ValueToDispose></ValueToDispose>" &
                            "<WriteDownCurrencyCode></WriteDownCurrencyCode>" &
                            "<WriteDown>100%</WriteDown>" &
                            "<SaleProceedsCurrencyCode></SaleProceedsCurrencyCode>" &
                            "<SaleProceeds></SaleProceeds>" &
                            "<SaleExpensesCurrencyCode></SaleExpensesCurrencyCode>" &
                            "<SaleExpenses>0.00</SaleExpenses>" &
                            "<SBID>7</SBID>" &
                            "<PurgeFlag>0</PurgeFlag>" &
                            "<Transfer></Transfer>" &
                            "<Transfer1></Transfer1>" &
                            "<Transfer2></Transfer2>" &
                            "<Transfer3></Transfer3>" &
                            "</BulkDisposal_R>" &
                            "</BulkDisposal_T>"

            Dim result As String = c.SaveSpecial("specialbulkdisposal", x, {})

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = "Disposal completed:" & result & vbCrLf & "Asset ID was " & _assetid & vbCrLf
            End If
        Catch ex As Exception
            HandleError(ex)
        End Try


    End Sub

    Private Sub DeleteAsset_Click(sender As Object, e As EventArgs) Handles DeleteAsset.Click

        Try

            Dim x As String =
                    "<BulkDisposal_T>" &
                    "<BulkDisposal_R>" &
                    "<a>1</a>" &
                    "<b>0</b>" &
                    "<D39>" & Now.ToString("dd-MMM-yyyy HH:mm:ss") & "</D39>" &
                    "<FixedList></FixedList>" &
                    "<AID>" & _assetid & "</AID>" &
                    "<SID>" & 6 & "</SID>" &        ' 6 is sold, but this and most other fields will be ignored on delete
                    "<BID>1</BID>" &
                    "<B1ID></B1ID>" &
                    "<ValueToDispose></ValueToDispose>" &
                    "<WriteDownCurrencyCode></WriteDownCurrencyCode>" &
                    "<WriteDown>100%</WriteDown>" &
                    "<SaleProceedsCurrencyCode></SaleProceedsCurrencyCode>" &
                    "<SaleProceeds></SaleProceeds>" &
                    "<SaleExpensesCurrencyCode></SaleExpensesCurrencyCode>" &
                    "<SaleExpenses>0.00</SaleExpenses>" &
                    "<SBID>7</SBID>" &
                    "<PurgeFlag>1</PurgeFlag>" &            '   THE ONLY DIFFERENCE BETWEEN DISPOSAL AND DELETE IS THIS FLAG
                    "<Transfer></Transfer>" &
                    "<Transfer1></Transfer1>" &
                    "<Transfer2></Transfer2>" &
                    "<Transfer3></Transfer3>" &
                    "</BulkDisposal_R>" &
                    "</BulkDisposal_T>"

            Dim result As String = c.SaveSpecial("specialbulkdisposal", x, {})

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = "Disposal completed:" & result & vbCrLf & "Asset ID was " & _assetid & vbCrLf
            End If

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub



    Private Sub GetMultipleAssets_Click(sender As Object, e As EventArgs) Handles GetMultipleAssets.Click

        '   Get the Assets with ID = 1,2,3,4,5

        Try
            '   Use an IN list to retrieve multiple asset records.

            Dim xml As String = c.CommandXML("AssetXML", {"LIST_ALL", "FIXEDLIST", "1,2,3,4,5"})

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = XMLToString(xml)
            End If

            _xml = xml

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub

    Private Sub GetCustom(sQueryString As String)

        '   command=AssetXML&arg0=10432&arg4=<param><AssetIDList>42</AssetIDList></param>&format=XML

        Dim nvc As Specialized.NameValueCollection = System.Web.HttpUtility.ParseQueryString(sQueryString)

        Dim command As String = ""
        Dim args(0 To 40) As String

        For Each name As String In nvc.Keys

            If name = "command" Then
                command = nvc.Get(name)
            ElseIf name.StartsWith("arg", StringComparison.CurrentCultureIgnoreCase) = True Then
                Dim c As String = name.Substring(3)
                If IsNumeric(c) = True Then
                    Dim index As Integer = CInt(c)
                    Dim a As String = nvc.Get(name)
                    If a.IndexOf("<") > -1 Then
                        a = XmlToParameters(a)
                    End If
                    args(index) = a
                End If
            End If

        Next

        If command <> "" Then
            Dim xml As String = c.CommandXML(command, args)

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = XMLToString(xml)
            End If

            _xml = xml

        End If


    End Sub

    Private Sub RunTransform_Click(sender As Object, e As EventArgs) Handles RunTransform.Click

        '   Run a transform which returns All Assets as a PDF file, and launch it into the browser

        Try

            '   Args(0) can be a transform name OR a transform ID. Use an ID where possible.

            Dim result As String = c.SaveSpecial("specialdbtransformrun", "<params><test>Location</test></params>", {"API Example Transform - Get All Assets PDF"})

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else

                Dim target As String = My.Application.Info.DirectoryPath & "\report.pdf"

                '   Remove special= from the result to get the path

                Dim webpath As String = c.DataPath & "/" & Mid(result, 9).Replace("\", "/")
                Dim w As New System.Net.WebClient()

                '   Download the actual file

                w.DownloadFile(webpath, target)

                '   And launch it with shell execute

                Dim i As New System.Diagnostics.ProcessStartInfo()
                i.FileName = "explorer"
                i.Arguments = """" & target & """"
                System.Diagnostics.Process.Start(i)

                InfoBox.Text = "PDF File " & target & " launched in a new window"

            End If

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub

    Private Sub GetAllAssets_Click(sender As Object, e As EventArgs) Handles GetAssetsAll.Click

        '   http://x7:58978/api/api.ashx?command=assetxml&username=apikey&database=x7&passwordx=SomePassword!!hh&arg1=10269&arg3=10269

        Try
            Dim xml As String = c.QueryXML("All Assets", "")

            If c.ErrorMessage <> "" Then
                InfoBox.Text = c.ErrorMessage
            Else
                InfoBox.Text = XMLToString(xml)
            End If

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub


    Private Sub GetAssetsJSON_Click(sender As Object, e As EventArgs) Handles GetAssetsJSON.Click

        '   http://x7:58978/api/api.ashx?command=assetxml&username=apikey&database=x7&passwordx=SomePassword!!hh&arg1=10269&arg3=10269

        Try
            If c.AccessType = "REST" Then
                Dim xml As String = c.Restful("", "json", "AssetXML", "", "10269")
                If c.ErrorMessage <> "" Then
                    InfoBox.Text = c.ErrorMessage
                Else
                    InfoBox.Text = xml
                End If
            Else
                InfoBox.Text = "This example only works with the restful API method"
            End If
        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub

    '------------------------------------
    '   PRIVATE
    '------------------------------------

    Private Function XMLToString(xml As String) As String

        Dim sb As New Text.StringBuilder
        Dim doc As New Xml.XmlDocument
        doc.LoadXml(xml)

        For Each nod As Xml.XmlNode In doc.DocumentElement.ChildNodes
            Dim i As Integer = 1
            sb.Append(i).Append(","c)
            For Each nodfield As Xml.XmlNode In nod.ChildNodes
                sb.Append(nodfield.InnerText).Append(","c)
            Next
            sb.Append(vbCrLf)
        Next

        Return sb.ToString

    End Function


    Public Sub New()

        ' This call is required by the designer.
        InitializeComponent()


    End Sub

    Private Sub MainForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load

        Try
            Dim index As String = CStr(My.Settings("Restful"))
            If index = "" Then
                cboAccessType.SelectedIndex = 0
            Else
                cboAccessType.SelectedIndex = CInt(index)
            End If
            Dim sWebsite As String = CStr(My.Settings("Website"))

            If sWebsite = "" Then
                sWebsite = "https://mycompany.hosted.xassets.net"
            End If

            txtWebsite.Text = sWebsite

            Dim user As String = CStr(My.Settings("Username"))
            If user = "" Then
                txtUsername.Text = System.Environment.UserDomainName & "\" & System.Environment.UserName
                txtPassword.Text = ""
            Else
                txtUsername.Text = user
                Dim pass As String = CStr(My.Settings("Password"))
                txtPassword.Text = pass
            End If

            txtCustom.Text = CStr(My.Settings("Custom"))

            WebsiteChanged()

        Catch ex As Exception
            HandleError(ex)
        End Try

    End Sub

    Private Sub cboAccessType_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cboAccessType.SelectedIndexChanged
        My.Settings("Restful") = cboAccessType.SelectedIndex
        WebsiteChanged()
        My.Settings.Save()
        EnableControls()
    End Sub

    Private Sub txtUsername_TextChanged(sender As Object, e As EventArgs) Handles txtUsername.TextChanged
        My.Settings("Username") = txtUsername.Text
        My.Settings.Save()
        WebsiteChanged()
    End Sub

    Private Sub txtPassword_TextChanged(sender As Object, e As EventArgs) Handles txtPassword.TextChanged
        My.Settings("Password") = txtPassword.Text
        My.Settings.Save()
        WebsiteChanged()
    End Sub


    Private Sub txtWebsite_TextChanged(sender As Object, e As EventArgs) Handles txtWebsite.TextChanged
        If txtWebsite.Text <> "website" And txtWebsite.Text <> "" Then
            My.Settings("Website") = txtWebsite.Text
            My.Settings.Save()
        End If
        WebsiteChanged()
    End Sub

    Private Sub txtCustom_TextChanged(sender As Object, e As KeyEventArgs) Handles txtCustom.KeyUp
        Try
            Select Case e.KeyCode
                Case Keys.Enter
                    My.Settings("Custom") = txtCustom.Text
                    GetCustom(txtCustom.Text)
                    e.Handled = True
                    e.SuppressKeyPress = True
            End Select

        Catch ex As Exception
            HandleError(ex)
        End Try
    End Sub


    Private Sub WebsiteChanged()

        Dim u As Uri = Nothing
        Uri.TryCreate(txtWebsite.Text, UriKind.Absolute, u)
        If u IsNot Nothing Then
            Dim sDNS As String
            If u.Host = "x7" Then
                sDNS = "localhost"
            Else
                sDNS = u.Host
            End If

            Dim db As String = u.Host.Split("."c)(0)
            If db = "localhost" Then        '   it's possible to have database=x7 when working locally, but this must still match the db of the API key so it's not hackable

                Dim q As String = u.Query
                Dim dic As Specialized.NameValueCollection = System.Web.HttpUtility.ParseQueryString(q)
                If dic("database") <> "" Then
                    db = dic("database")
                End If

            End If

            c = New Connector(db, u.Port, sDNS, u.Scheme, u.LocalPath, txtUsername.Text)

            c.AccessType = Strings.Left(cboAccessType.Text, 4)

        End If
        EnableControls()

    End Sub

    Private Function XmlToParameters(s As String) As String

        Dim doc As New Xml.XmlDocument
        doc.LoadXml(s)

        Dim sbout As New Text.StringBuilder("p?")

        For Each nod As Xml.XmlNode In doc.DocumentElement.ChildNodes
            Dim sname As String = nod.Name
            Dim svalue As String = nod.InnerText
            sbout.Append(sname & "=" & svalue & "&")
        Next

        Return sbout.ToString

    End Function

    Private Sub HandleError(ex As Exception)

        Dim s As String = ex.Message

        If ex.InnerException IsNot Nothing Then
            s &= vbCrLf & vbCrLf & ex.InnerException.Message
        End If

        MsgBox("Error:" & s, MsgBoxStyle.Exclamation Or MsgBoxStyle.OkOnly, "xAssets API Examples")

    End Sub

    Private Sub GetForm_Click(sender As Object, e As EventArgs) Handles GetForm.Click

        Try
            If c.AccessType = "REST" Then
                Dim xml As String = c.Restful(
                    "all", "json", "form", "",
                    "form=jira asset panel&assetid=1")
                If c.ErrorMessage <> "" Then
                    InfoBox.Text = c.ErrorMessage
                Else
                    InfoBox.Text = xml
                End If
            Else
                InfoBox.Text = "The get form example only works with the restful API method"
            End If
        Catch ex As Exception
            HandleError(ex)
        End Try


    End Sub
End Class

